package com.hungraim.ltc.facelocation.util;

import com.alibaba.fastjson.JSON;
import com.hungraim.ltc.dao.CallFaceInterfaceLogMapper;
import com.hungraim.ltc.pojo.entity.disable.CallFaceInterfaceLog;
import com.hungraim.ltc.pojo.entity.serviceManage.ChSrvOrganEmp;
import com.tencentcloudapi.common.AbstractModel;
import com.tencentcloudapi.common.Credential;
import com.tencentcloudapi.common.profile.ClientProfile;
import com.tencentcloudapi.common.profile.HttpProfile;
import com.tencentcloudapi.iai.v20200303.IaiClient;
import com.tencentcloudapi.iai.v20200303.models.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Configuration;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.Base64;
import java.util.HashMap;
import java.util.Map;

import com.hungraim.ltc.util.Result;
/**
 * <pre>
 * 人脸核身代码
 * </pre>
 */
@Slf4j
@Configuration
public class FaceAuthUtil {

    @Autowired
    private FaceAuthProperties faceAuthProperties;

    @Autowired
    private CallFaceInterfaceLogMapper callFaceInterfaceLogMapper;

    public Result<String> addFace(ChSrvOrganEmp faceAuthParam,String filePath,Integer userId){

         try {

             log.info("filePath4:"+filePath);
             //设置文件路径
             // 实例化一个认证对象，入参需要传入腾讯云账户 SecretId 和 SecretKey
             Credential cred = new Credential(faceAuthProperties.getSecretId(), faceAuthProperties.getSecretKey());
             // 实例化一个http选项，可选的，没有特殊需求可以跳过
             HttpProfile httpProfile = new HttpProfile();
             httpProfile.setEndpoint(faceAuthProperties.getUrlPath());
             // 实例化一个client选项，可选的，没有特殊需求可以跳过
             ClientProfile clientProfile = new ClientProfile();
             clientProfile.setHttpProfile(httpProfile);
             // 实例化要请求产品的client对象,clientProfile是可选的
             IaiClient client = new IaiClient(cred, faceAuthProperties.getRegion(), clientProfile);
             // 实例化一个请求对象,每个接口都会对应一个request对象
             CreatePersonRequest req = new CreatePersonRequest();
             req.setGroupId(faceAuthProperties.getGroupId());
             req.setPersonName(faceAuthParam.getEmpName());
             req.setPersonId(faceAuthParam.getCertifNum());
             if(faceAuthParam.getGender().equals("M")){
                 req.setGender(1l);
             }else if(faceAuthParam.getGender().equals("F")){
                 req.setGender(2l);
             }else{
                 req.setGender(0l);
             }
             log.info("personName:::::"+faceAuthParam.getEmpName());
             log.info("personId:::::"+faceAuthParam.getCertifNum());
             log.info("gender:::::"+req.getGender());
             log.info("照片base:::::"+imageToBase64(filePath));
             req.setImage(imageToBase64(filePath));

             Map map=new HashMap();//记录调用接口传的参数
             map.put("groupId",faceAuthProperties.getGroupId());
             map.put("personName",req.getPersonName());
             map.put("personId",req.getPersonId());
             map.put("gender",req.getGender());

             CreatePersonResponse resp = client.CreatePerson(req);
             log.info("腾讯云返回:"+AbstractModel.toJsonString(resp));

             try {
                 //增加接口调用记录
                 addCallFaceInterfaceLog(userId, JSON.toJSONString(map), "CreatePerson", "创建人员",
                         AbstractModel.toJsonString(resp), resp.getFaceId(), null, null);
             }catch(Exception e){
                 log.error("增加CreatePerson接口调用记录失败："+e.toString());
             }
             return Result.success(resp.getFaceId());
            } catch (Exception e) {
                 log.error("调用腾讯云创建人员接口返回失败："+e.toString());
                 log.error(e.toString());
                 try{
                     //增加接口调用记录
                     addCallFaceInterfaceLog(userId, JSON.toJSONString(faceAuthParam.getFaceId()),"CreatePerson","创建人员",
                             e.getMessage(),null,null,null);
                 }catch(Exception e1){
                     log.error("增加CreatePerson接口调用记录失败2："+e1.toString());
                 }
                 return Result.failed(e.getMessage());
            }
    }

    public  String imageToBase64(String imagePath) {
        String base64String = "";
        //如果上传报错，本地测试将这两行打开
        //同时将FileUtils类uploadFile方法注释打开
       // File filePath=new File("D:/");
       // File file = new File(filePath,imagePath);
        log.info("imagePath:"+imagePath);
        File file = new File(imagePath);
        log.info("imageToBase64:"+file.getPath());
        try (FileInputStream fileInputStream = new FileInputStream(file)) {
            byte[] bytes = new byte[(int) file.length()];
            log.info("bytes:"+bytes.length);
            fileInputStream.read(bytes);
            base64String = Base64.getEncoder().encodeToString(bytes);
        } catch (IOException e) {
            e.printStackTrace();
        }
        return base64String;
    }



    public  Result<String> deleteFace(ChSrvOrganEmp faceAuthParam,Integer userId){

        try {
            // 实例化一个认证对象，入参需要传入腾讯云账户 SecretId 和 SecretKey
            Credential cred = new Credential(faceAuthProperties.getSecretId(), faceAuthProperties.getSecretKey());
            // 实例化一个http选项，可选的，没有特殊需求可以跳过
            HttpProfile httpProfile = new HttpProfile();
            httpProfile.setEndpoint(faceAuthProperties.getUrlPath());

            // 实例化一个client选项，可选的，没有特殊需求可以跳过
            ClientProfile clientProfile = new ClientProfile();
            clientProfile.setHttpProfile(httpProfile);
            // 实例化要请求产品的client对象,clientProfile是可选的
            IaiClient client = new IaiClient(cred, faceAuthProperties.getRegion(), clientProfile);
            // 实例化一个请求对象,每个接口都会对应一个request对象
            DeletePersonRequest  req = new DeletePersonRequest ();
            req.setPersonId(faceAuthParam.getCertifNum());

            // 返回的resp是一个DeleteFaceResponse的实例，与请求对象对应
            DeletePersonResponse resp = client.DeletePerson(req);
            // 输出json格式的字符串回包
            System.out.println(AbstractModel.toJsonString(resp));
            try{
                //增加接口调用记录
                //不收费，统计收费接口次数时无需统计这个接口
                addCallFaceInterfaceLog(userId, req.getPersonId(),"DeleteFace","删除人脸",
                        AbstractModel.toJsonString(resp),null,null,null);
            }catch(Exception e){
                log.error("增加DeleteFace接口调用记录失败："+e.toString());
            }
            // 输出json格式的字符串回包
            return Result.success(resp.getRequestId());

        } catch (Exception e) {
            try{
                //增加接口调用记录
                addCallFaceInterfaceLog(userId, faceAuthParam.getCertifNum(),"DeleteFace","删除人脸",
                      e.getMessage(),null,null,null);
            }catch(Exception e1){
                log.error("增加DeleteFace接口调用记录失败2："+e1.toString());
            }
            log.error("调用腾讯云删除人脸接口返回失败："+e.toString());
            return Result.failed(e.getMessage());
        }
    }

    //记录调用日志
    private void addCallFaceInterfaceLog(Integer userId,String callParam,String interfaceMethod,
                                         String interfaceName,String callResult,
                                         String faceId,String isMatch,String score){
        CallFaceInterfaceLog callFaceInterfaceLog=new CallFaceInterfaceLog();
        callFaceInterfaceLog.setPcUserId(userId);
        callFaceInterfaceLog.setCallParam(callParam);
        callFaceInterfaceLog.setInterfaceMethod(interfaceMethod);
        callFaceInterfaceLog.setInterfaceName(interfaceName);
        callFaceInterfaceLog.setCallResult(callResult);
        callFaceInterfaceLog.setFaceId(faceId);
        callFaceInterfaceLog.setIsMatch(isMatch);
        callFaceInterfaceLog.setScore(score);
        callFaceInterfaceLogMapper.insert(callFaceInterfaceLog);
    }
}
